SET TERM ^ ;
                         
CREATE TRIGGER POSITIONS_BI0 FOR POSITIONS
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  DELETE FROM POSITIONS
  WHERE LOGIN = NEW.LOGIN
  AND KEYFIELD = NEW.KEYFIELD            
  AND SECTIONFIELD = NEW.SECTIONFIELD
  AND MANDAT_IDT = NEW.MANDAT_IDT; 
END^

SET TERM ; ^


DROP PROCEDURE GETCOMPTESOLDE; 
DROP PROCEDURE GETCOMPTESOLDE_;

SET TERM ^ ;

CREATE PROCEDURE GETCOMPTESOLDE_ (
 EXERCICE_IDT         INTEGER,
 CPT_DE               VARCHAR(15),
 CPT_A                VARCHAR(15),
 DT_DEB               DATE,
 DT_FIN               DATE,
 PLAN_COMPTABLE       VARCHAR(15),
 SLD_ZERO_HIDDEN      SMALLINT,
 EXERCICE_IDT_PREC1   INTEGER,
 CLASSE_FROM          SMALLINT,
 CLASSE_TO            SMALLINT,
 WITH_GROUPED         SMALLINT = 0,
 WITH_TITRE           SMALLINT = 1)
RETURNS (
 CPTE_ID              VARCHAR(15),
 INTITULE             VARCHAR(120),
 TITRE                INTEGER,
 CODE_MONNAIE         VARCHAR(3),
 CLASSE               VARCHAR(1),
 REFLECHIS            VARCHAR(15),
 CPT_GROUPEMENT       VARCHAR(15),
 PARENT               VARCHAR(15),
 NIVEAU               SMALLINT,
 HIDDEN               SMALLINT,
 SOLDE_INITIAL        NUMERIC(15,4),
 SOLDE_INITIAL_ME     NUMERIC(15,4),
 SOLDE_INI_GRP        NUMERIC(15,4),
 SOLDE_INI_ME_GRP     NUMERIC(15,4),
 SOLDE_INI_R          NUMERIC(15,4),
 SOLDE_INI_ME_R       NUMERIC(15,4),
 SOLDE_DB             NUMERIC(15,2),
 SOLDE_CR             NUMERIC(15,2),
 SOLDE_DB_ME          NUMERIC(15,2),
 SOLDE_CR_ME          NUMERIC(15,2),
 SOLDE_DB_GRP         NUMERIC(15,2),
 SOLDE_CR_GRP         NUMERIC(15,2),
 SOLDE_DB_GRP_ME      NUMERIC(15,2),
 SOLDE_CR_GRP_ME      NUMERIC(15,2),
 SOLDE_DB_R           NUMERIC(15,2),
 SOLDE_CR_R           NUMERIC(15,2),
 SOLDE_DB_R_ME        NUMERIC(15,2),
 SOLDE_CR_R_ME        NUMERIC(15,2),
 BUDGET               NUMERIC(15,2),
 SOLDE_FINAL          NUMERIC(15,2),
 SOLDE_FINAL_ME       NUMERIC(15,2),
 SOLDE_FINAL_PREC1    NUMERIC(15,2),
 SOLDE_FINAL_ME_PREC1 NUMERIC(15,2),
 BUDGET_PREC1         NUMERIC(15,2))
AS 
DECLARE VARIABLE cpt_tmp VARCHAR(15);
DECLARE VARIABLE tmp_dt_deb DATE;
DECLARE VARIABLE tmp_dt_fin DATE;
DECLARE VARIABLE solde_ini_grp_tmp NUMERIC(15, 4);
DECLARE VARIABLE solde_ini_me_grp_tmp NUMERIC(15, 4);
DECLARE VARIABLE solde_db_grp_tmp NUMERIC(15, 2);
DECLARE VARIABLE solde_cr_grp_tmp NUMERIC(15, 2);
DECLARE VARIABLE solde_db_grp_me_tmp NUMERIC(15, 2);
DECLARE VARIABLE solde_cr_grp_me_tmp NUMERIC(15, 2);
DECLARE VARIABLE pos INTEGER;
DECLARE VARIABLE solde_db_r_tmp NUMERIC(15, 2);
DECLARE VARIABLE solde_cr_r_tmp NUMERIC(15, 2);
DECLARE VARIABLE solde_db_me_r_tmp NUMERIC(15, 2);
DECLARE VARIABLE solde_cr_me_r_tmp NUMERIC(15, 2);
DECLARE VARIABLE solde_ini_r_tmp NUMERIC(15, 4);
DECLARE VARIABLE solde_ini_me_r_tmp NUMERIC(15, 4);
DECLARE VARIABLE solde_final_r_tmp NUMERIC(15, 2);
DECLARE VARIABLE shw SMALLINT;
begin
    --idem pour les date des ecritures
    SELECT DATE_DEBUT, DATE_FIN
    FROM EXERCICE
    WHERE exercice_idt = :exercice_idt
    INTO :tmp_dt_deb, :tmp_dt_fin;

    IF (coalesce(:DT_DEB, '1899-12-30') = '1899-12-30') THEN BEGIN DT_DEB = :tmp_dt_deb; END
    IF (coalesce(:DT_FIN, '1899-12-30') = '1899-12-30') THEN BEGIN DT_FIN = :tmp_dt_fin; END
    
    FOR
        SELECT  p.pos, c.cpte_id, c.intitule, c.titre, c.code_monnaie, c.classe, c.budget,
                c.reflechis, c.cpt_groupement, c.solde_initial, c.solde_initial_me,
                p.parent, p.niveau, p.hidden,
                coalesce(SUM(coalesce(e.debit, 0)), 0), coalesce(SUM(coalesce(e.CREDIT, 0)), 0), coalesce(SUM(coalesce(e.debit_me, 0)), 0), coalesce(SUM(coalesce(e.CREDIT_ME, 0)), 0)

        FROM comptes c
        
        LEFT JOIN PLAN_COMPTABLE P
        ON P.exercice_idt = c.exercice_idt
        AND P.compte = c.cpte_id
        AND P.pc_id = :plan_comptable
        
        LEFT JOIN ECRITURES e
        ON e.exercice_idt = c.exercice_idt
        AND e.cpte = c.cpte_id
        AND e.DATE_ECR BETWEEN :dt_deb AND :dt_fin
        
        WHERE c.exercice_idt = :exercice_idt
        AND c.cpte_id BETWEEN :CPT_DE AND :CPT_A
        AND ( (:PLAN_COMPTABLE <> '') OR (:WITH_GROUPED = 1) OR ((:WITH_GROUPED=0) AND (coalesce(c.cpt_groupement, '') = '')) )        
        AND ( (:WITH_TITRE=1) OR ((:WITH_TITRE=0) AND (c.TITRE = 0)) )
        AND c.classe BETWEEN :CLASSE_FROM AND :CLASSE_TO

        GROUP BY 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14
        ORDER BY 1, 2

        INTO    :POS, :CPTE_ID, :INTITULE, :TITRE, :CODE_MONNAIE, :CLASSE, :BUDGET,
                :REFLECHIS, :CPT_GROUPEMENT, :SOLDE_INITIAL, :SOLDE_INITIAL_ME, :PARENT,
                :NIVEAU, :HIDDEN, :SOLDE_DB, :SOLDE_CR, :SOLDE_DB_ME, :SOLDE_CR_ME

    DO BEGIN
          solde_ini_grp       = 0;
          solde_ini_me_grp    = 0;
          solde_db_grp        = 0;
          solde_cr_grp        = 0;
          solde_db_grp_me     = 0;
          solde_cr_grp_me     = 0;
          solde_db_r          = 0;
          solde_cr_r          = 0;
          solde_db_r_me       = 0;
          solde_cr_r_me       = 0;
          solde_ini_r         = 0;
          solde_ini_me_r      = 0;
          CPT_TMP             = '';
    
          --COMPTES REFLECHIS
          FOR
              SELECT cpte_id
              FROM COMPTES
              WHERE exercice_idt = :exercice_idt
              AND reflechis = :CPTE_ID
              INTO :CPT_TMP
          DO BEGIN
              SELECT SUM(solde_initial), SUM(solde_initial_me),
              SUM(solde_db), SUM(solde_cr), SUM(solde_db_me), SUM(solde_cr_me),
              SUM(SOLDE_FINAL)
              FROM GETCOMPTESOLDE_(:EXERCICE_IDT, :CPT_TMP, :CPT_TMP, :DT_DEB, :DT_FIN, :plan_comptable, 0, 0, null, null)
              INTO :solde_ini_r_tmp, :solde_ini_me_r_tmp, :solde_db_r_tmp, :solde_cr_r_tmp,
              :solde_db_me_r_tmp, :solde_cr_me_r_tmp, :SOLDE_FINAL_R_TMP;
    
              --ne pas ajouter les montants si pas negatif
              if ( :CLASSE in ('1', '3') ) then begin
                  if ( :SOLDE_FINAL_R_TMP > 0 ) then begin
                      solde_ini_r     = :solde_ini_r + :solde_ini_r_tmp;
                      solde_ini_me_r  = :solde_ini_me_r + :solde_ini_me_r_tmp;
                      solde_db_r      = :solde_db_r + :solde_db_r_tmp;
                      solde_cr_r      = :solde_cr_r + :solde_cr_r_tmp;
                      solde_db_r_me   = :solde_db_r_me + :solde_db_me_r_tmp;
                      solde_cr_r_me   = :solde_cr_r_me + :solde_cr_me_r_tmp;
                  end
              end else begin
                  if ( :SOLDE_FINAL_R_TMP < 0 ) then begin
                      solde_ini_r     = :solde_ini_r + :solde_ini_r_tmp;
                      solde_ini_me_r  = :solde_ini_me_r + :solde_ini_me_r_tmp;
                      solde_db_r      = :solde_db_r + :solde_db_r_tmp;
                      solde_cr_r      = :solde_cr_r + :solde_cr_r_tmp;
                      solde_db_r_me   = :solde_db_r_me + :solde_db_me_r_tmp;
                      solde_cr_r_me   = :solde_cr_r_me + :solde_cr_me_r_tmp;
                  end
              end
          END
    
          --COMPTES REGROUPES
          IF ((:PLAN_COMPTABLE = '') AND (:WITH_GROUPED=0)) THEN BEGIN
             FOR
                           SELECT cpte_id, coalesce(solde_initial, 0), coalesce(solde_initial_me, 0)
                           FROM COMPTES
                           WHERE exercice_idt = :exercice_idt
                           AND cpt_groupement = :CPTE_ID
                           INTO :CPT_TMP, :solde_ini_grp_tmp, :solde_ini_me_grp_tmp
              DO BEGIN
                 solde_ini_grp       = :solde_ini_grp + :solde_ini_grp_tmp;
                 solde_ini_me_grp    = :solde_ini_me_grp + :solde_ini_me_grp_tmp;
    
                 SELECT coalesce(SUM(coalesce(debit, 0)), 0), coalesce(SUM(coalesce(credit, 0)), 0), coalesce(SUM(coalesce(debit_me, 0)), 0), coalesce(SUM(coalesce(credit_me, 0)), 0)
                 FROM ECRITURES
                 WHERE exercice_idt = :exercice_idt
                 AND cpte = :CPT_TMP
                 AND DATE_ECR BETWEEN :dt_deb AND :dt_fin
                 INTO :solde_db_grp_tmp, :solde_cr_grp_tmp, :solde_db_grp_me_tmp, :solde_cr_grp_me_tmp;
    
                 solde_db_grp        = solde_db_grp + solde_db_grp_tmp;
                 solde_cr_grp        = solde_cr_grp + solde_cr_grp_tmp;
                 solde_db_grp_me     = solde_db_grp_me + solde_db_grp_me_tmp;
                 solde_cr_grp_me     = solde_cr_grp_me + solde_cr_grp_me_tmp;
               END
          END
    
          --ORDRE D'IMPRESSION PAR DEFAUT SI PAS RENSEIGNE
          IF (coalesce(:plan_comptable, '') = '') THEN BEGIN
              SELECT FIRST 1 CPTE_ID, CHAR_LENGTH(CPTE_ID)
              FROM comptes
              WHERE exercice_idt = :exercice_idt
              AND CHAR_LENGTH(cpte_id) < CHAR_LENGTH(:CPTE_ID)
              AND cpte_id < :CPTE_ID
              AND titre = 1
              AND classe = :CLASSE 
              ORDER BY cpte_id DESC
              INTO :PARENT, :NIVEAU;
    
              PARENT = coalesce(:PARENT, '');
              NIVEAU = coalesce(:NIVEAU, 0);
          END
    
          --si je suis ACTIF, mon solde doit etre positif
          --donc le solde de mon compte reflechis devrait etre negatif pour moi
          --s'il est positif pour moi, c'est que je dois l'afficher  
          SOLDE_FINAL        = :solde_ini_r + :solde_db_r - :solde_cr_r +
                               :solde_initial + :solde_db - :solde_cr +
                               :solde_ini_grp + :solde_db_grp - :solde_cr_grp;
    
          SOLDE_FINAL_ME     = :solde_ini_me_r + :solde_db_r_me - :solde_cr_r_me +
                               :solde_initial_me + :solde_db_me - :solde_cr_me +
                               :solde_ini_me_grp + :solde_db_grp_me - :solde_cr_grp_me;
    
          --solde de l'exercice precedent
          if (:EXERCICE_IDT_PREC1 > 0) then begin
              SELECT SUM(SOLDE_FINAL), SUM(SOLDE_FINAL_ME), SUM(BUDGET)
              FROM GETCOMPTESOLDE_(:EXERCICE_IDT_PREC1, :CPTE_ID, :CPTE_ID, null, null, :plan_comptable, 0, 0, 1, 9)
              INTO :SOLDE_FINAL_PREC1, :SOLDE_FINAL_ME_PREC1, :BUDGET_PREC1;
          end
    
          SHW = 0;
          if ( (:SLD_ZERO_HIDDEN = 1) AND (:titre <> 1) ) then begin
              if ((:SOLDE_FINAL <> 0) or (:SOLDE_FINAL_ME <> 0) or (:SOLDE_FINAL_PREC1 <> 0) or (:SOLDE_FINAL_ME_PREC1 <> 0) ) then begin
                  SHW = 1;
              end
          end else begin
              SHW = 1;
          end
    
          if (:SHW = 1) then begin
              --s'il n'est pas reflechis sur un autre, l'afficher sans autre
              if (coalesce(:REFLECHIS, '') = '') then begin
                  suspend;
              end else begin
                  --sinon l'afficher selon son signe
                  if ( :CLASSE in ('1', '3') ) then begin
                      if ( :SOLDE_FINAL > 0 ) then begin
                          suspend;
                      end
                  end else begin
                      if ( :SOLDE_FINAL < 0 ) then begin
                          suspend;
                      end
                  end
              end
          end
     end
end^

SET TERM ; ^


SET TERM ^ ;

CREATE PROCEDURE GETCOMPTESOLDE (
 EXERCICE_IDT         INTEGER,
 CPT_DE               VARCHAR(15),
 CPT_A                VARCHAR(15),
 DT_DEB               DATE,
 DT_FIN               DATE,
 PLAN_COMPTABLE       VARCHAR(15),
 SLD_ZERO_HIDDEN      SMALLINT,
 EXERCICE_IDT_PREC1   INTEGER,
 CLASSE_FROM          SMALLINT,
 CLASSE_TO            SMALLINT,
 WITH_GROUPED         SMALLINT = 0,
 WITH_TITRE           SMALLINT = 1)
RETURNS (
 CPTE_ID              VARCHAR(15),
 INTITULE             VARCHAR(120),
 TITRE                INTEGER,
 CODE_MONNAIE         VARCHAR(3),
 CLASSE               VARCHAR(1),
 REFLECHIS            VARCHAR(15),
 CPT_GROUPEMENT       VARCHAR(15),
 PARENT               VARCHAR(15),
 NIVEAU               SMALLINT,
 HIDDEN               SMALLINT,
 SOLDE_INITIAL        NUMERIC(15,4),
 SOLDE_INITIAL_ME     NUMERIC(15,4),
 SOLDE_INI_GRP        NUMERIC(15,4),
 SOLDE_INI_ME_GRP     NUMERIC(15,4),
 SOLDE_INI_R          NUMERIC(15,4),
 SOLDE_INI_ME_R       NUMERIC(15,4),
 SOLDE_DB             NUMERIC(15,2),
 SOLDE_CR             NUMERIC(15,2),
 SOLDE_DB_ME          NUMERIC(15,2),
 SOLDE_CR_ME          NUMERIC(15,2),
 SOLDE_DB_GRP         NUMERIC(15,2),
 SOLDE_CR_GRP         NUMERIC(15,2),
 SOLDE_DB_GRP_ME      NUMERIC(15,2),
 SOLDE_CR_GRP_ME      NUMERIC(15,2),
 SOLDE_DB_R           NUMERIC(15,2),
 SOLDE_CR_R           NUMERIC(15,2),
 SOLDE_DB_R_ME        NUMERIC(15,2),
 SOLDE_CR_R_ME        NUMERIC(15,2),
 BUDGET               NUMERIC(15,2),
 SOLDE_FINAL          NUMERIC(15,2),
 SOLDE_FINAL_ME       NUMERIC(15,2),
 SOLDE_FINAL_PREC1    NUMERIC(15,2),
 SOLDE_FINAL_ME_PREC1 NUMERIC(15,2),
 BUDGET_PREC1         NUMERIC(15,2))
AS 
DECLARE VARIABLE tmp_cpte_de VARCHAR(15);
DECLARE VARIABLE tmp_cpte_a VARCHAR(15);
DECLARE VARIABLE tmp_cpte_titre VARCHAR(15);
DECLARE VARIABLE tmp_view_name VARCHAR(30);
DECLARE VARIABLE tmp_id INTEGER;
DECLARE VARIABLE solde_final_tmp NUMERIC(15, 2);
DECLARE VARIABLE solde_final_me_tmp NUMERIC(15, 2);
DECLARE VARIABLE solde_final_prec1_tmp NUMERIC(15, 2);
DECLARE VARIABLE solde_final_me_prec1_tmp NUMERIC(15, 2);
begin
    --nom de la vue temporaire
    SELECT GEN_ID(GEN_TMP_GETCOMPTESOLDE, 1)
    FROM rdb$database
    INTO :tmp_id;
    
    --renseigner les filtres cpt_de et cpt_a si non communiques
    select min(cpte_id), max(cpte_id)
    from comptes
    where exercice_idt = :exercice_idt
    into :tmp_cpte_de, :tmp_cpte_a;

    IF (coalesce(:CPT_DE, '') = '') THEN BEGIN CPT_DE = :tmp_cpte_de; END
    IF (coalesce(:CPT_A, '') = '') THEN BEGIN CPT_A = :tmp_cpte_a; END

    IF (coalesce(:CLASSE_FROM, 0) = 0) THEN BEGIN CLASSE_FROM = 1; END
    IF (coalesce(:CLASSE_TO, 0) = 0) THEN BEGIN CLASSE_TO = 9; END    

    --inserer dans une table temporaire les montants des comptes sans les titres
    INSERT INTO TMP_GETCOMPTESOLDE
    SELECT :tmp_id, cpte_id, intitule, titre, code_monnaie, classe, reflechis, cpt_groupement, parent,
    niveau, hidden, solde_initial, solde_initial_me, solde_ini_grp, solde_ini_me_grp, solde_ini_r,
    solde_ini_me_r, solde_db, solde_cr, solde_db_me, solde_cr_me, solde_db_grp, solde_cr_grp,
    solde_db_grp_me, solde_cr_grp_me, solde_db_r, solde_cr_r, solde_db_r_me, solde_cr_r_me, budget,
    solde_final, solde_final_me, solde_final_prec1, solde_final_me_prec1, budget_prec1  
    FROM GETCOMPTESOLDE_(:EXERCICE_IDT, :CPT_DE, :CPT_A, :DT_DEB, :DT_FIN, :PLAN_COMPTABLE, :SLD_ZERO_HIDDEN, :EXERCICE_IDT_PREC1, :CLASSE_FROM, :CLASSE_TO, :WITH_GROUPED, :WITH_TITRE);
    
    --travailler sur les titres
    FOR
        SELECT cpte_id
        FROM TMP_GETCOMPTESOLDE
        WHERE id = :tmp_id
        AND titre = 1
        ORDER BY cpte_id desc
        INTO :tmp_cpte_titre
    DO BEGIN
      if (coalesce(:PLAN_COMPTABLE, '') = '') then begin     
          for select sum(SOLDE_FINAL), sum(SOLDE_FINAL_ME), sum(SOLDE_FINAL_PREC1), sum(SOLDE_FINAL_ME_PREC1)
              from TMP_GETCOMPTESOLDE
              where id = :tmp_id
              and parent = :tmp_cpte_titre
              into :solde_final_tmp, :solde_final_me_tmp, :solde_final_prec1_tmp, :solde_final_me_prec1_tmp
          do
              update TMP_GETCOMPTESOLDE
              set solde_final = :solde_final_tmp,
              solde_final_me = :solde_final_me_tmp,
              solde_final_prec1 = :solde_final_prec1_tmp,
              solde_final_me_prec1 = :solde_final_me_prec1_tmp
              where id = :tmp_id
              and CPTE_ID = :tmp_cpte_titre;
      end else begin
            --un plan comptable a ete specifie
          for select sum(t.SOLDE_FINAL), sum(t.SOLDE_FINAL_ME), sum(t.SOLDE_FINAL_PREC1), sum(t.SOLDE_FINAL_ME_PREC1)
              from TMP_GETCOMPTESOLDE t

              inner join GETCOMPTECHILD(:exercice_idt, :tmp_cpte_titre, :PLAN_COMPTABLE) as c
              on c.compte = t.cpte_id
              and c.titre = 0
              
              where t.id = :tmp_id
              and t.titre = 0
              into :solde_final_tmp, :solde_final_me_tmp, :solde_final_prec1_tmp, :solde_final_me_prec1_tmp

          do
              update TMP_GETCOMPTESOLDE
              set solde_final = :solde_final_tmp,
              solde_final_me = :solde_final_me_tmp,
              solde_final_prec1 = :solde_final_prec1_tmp,
              solde_final_me_prec1 = :solde_final_me_prec1_tmp
              where id = :tmp_id
              and CPTE_ID = :tmp_cpte_titre;
      end
    end
    
 
    if (:SLD_ZERO_HIDDEN = 1)then begin
        DELETE FROM TMP_GETCOMPTESOLDE
        WHERE ID = :tmp_id
        AND TITRE = 1
        AND (    (coalesce(SOLDE_FINAL, 0) = 0) and
                (coalesce(SOLDE_FINAL_ME, 0) = 0) and
                (coalesce(SOLDE_FINAL_PREC1, 0) = 0) and
                (coalesce(SOLDE_FINAL_ME_PREC1, 0) = 0) );
    end

    FOR 
        SELECT cpte_id, intitule, titre, code_monnaie, classe, reflechis, cpt_groupement, parent,
        niveau, hidden, solde_initial, solde_initial_me, solde_ini_grp, solde_ini_me_grp, solde_ini_r,
        solde_ini_me_r, solde_db, solde_cr, solde_db_me, solde_cr_me, solde_db_grp, solde_cr_grp,
        solde_db_grp_me, solde_cr_grp_me, solde_db_r, solde_cr_r, solde_db_r_me, solde_cr_r_me, budget,
        solde_final, solde_final_me, solde_final_prec1, solde_final_me_prec1, budget_prec1
        FROM TMP_GETCOMPTESOLDE
        WHERE ID = :tmp_id
               
        INTO :cpte_id, :intitule, :titre, :code_monnaie, :classe, :reflechis, :cpt_groupement, :parent,
        :niveau, :hidden, :solde_initial, :solde_initial_me, :solde_ini_grp, :solde_ini_me_grp, :solde_ini_r,
        :solde_ini_me_r, :solde_db, :solde_cr, :solde_db_me, :solde_cr_me, :solde_db_grp, :solde_cr_grp,
        :solde_db_grp_me, :solde_cr_grp_me, :solde_db_r, :solde_cr_r, :solde_db_r_me, :solde_cr_r_me, :budget,
        :solde_final, :solde_final_me, :solde_final_prec1, :solde_final_me_prec1, :budget_prec1
    DO BEGIN
        suspend;
    END
    
    DELETE FROM TMP_GETCOMPTESOLDE
    WHERE ID = :tmp_id;
end^

SET TERM ; ^

DROP TRIGGER ECRITURES_ANALYTIQUE_AI;
DROP PROCEDURE ANALYTIQUE_GET_PERIODE_DATE;

SET TERM ^ ;

CREATE PROCEDURE ANALYTIQUE_GET_PERIODE_DATE (
 MANDAT_IDT        INTEGER,
 DATE_ECR          DATE,
 PERIODE           INTEGER)
RETURNS (
 REAL_PERIODE_DATE DATE)             
AS 
declare ecr_real_period integer;
declare start_period_year integer;
declare start_period_month integer;
declare period_cycle_month integer;
declare diff_avant integer;
declare diff_apres integer;
BEGIN
     SELECT START_PERIOD_MONTH, START_PERIOD_YEAR, PERIOD_CYCLE_MONTH
     FROM MANDAT              
     WHERE MANDAT_IDT = :MANDAT_IDT
     INTO :start_period_month, :start_period_year, :period_cycle_month;
     
     if (:period_cycle_month = 0) then begin
        REAL_PERIODE_DATE = null;
        suspend;
        exit;
     end

     --calculer la periode en rapport a la date de l'ecriture (celle qui est passee peut etre differente)
     ecr_real_period = (EXTRACT(YEAR from :date_ecr)-:start_period_year)*12 + (EXTRACT(MONTH from :date_ecr) - :start_period_month);
     ecr_real_period = MOD(:ecr_real_period, :period_cycle_month) + 1;
          
     --date du premier jour du mois de l'ecritures passee                
     REAL_PERIODE_DATE = dateadd(day, EXTRACT(DAY from :date_ecr)*-1+1, :date_ecr);
              
     --prendre la periode la plus proche comme reference
              
     --d'abord calculer la difference avec la periode precedente
     diff_avant = (:periode - :ecr_real_period)*-1;
              
     --si negatif, un cycle est depasse
     if (diff_avant < 0) then begin
        diff_avant = (:period_cycle_month + :diff_avant);
     end
              
     --calculer la difference avec la periode precedente
     diff_apres = (:periode - :ecr_real_period);
              
     --si negatif, un cycle est depasse
     if (diff_apres < 0) then begin
        diff_apres = (:period_cycle_month + :diff_apres);
     end

     --toujours prendre la difference la plus petite ou la periode precedente en cas d'egalite
     if (diff_avant > diff_apres) then begin
        REAL_PERIODE_DATE = dateadd(month, diff_apres, :REAL_PERIODE_DATE);                    
     end else begin
        REAL_PERIODE_DATE = dateadd(month, diff_avant*-1, :REAL_PERIODE_DATE);
     end
     suspend; 
END^

SET TERM ; ^

SET TERM ^ ;

CREATE TRIGGER ECRITURES_ANALYTIQUE_AI FOR ECRITURES
ACTIVE AFTER INSERT POSITION 0
AS
declare i integer;
declare PERIOD_CYCLE_MONTH integer;
declare REAL_PERIODE_DATE date;
declare ANALYTIQUE_AUTO integer;

begin   
  if (coalesce(new.cpte_analyse, '') <> '') then begin                --si une analyse a enregistrer
      --############# REEL DATE DE LA PERIODE ANALYTIQUE INDIQUEE
      SELECT REAL_PERIODE_DATE          
      FROM ANALYTIQUE_GET_PERIODE_DATE((SELECT MANDAT_IDT FROM EXERCICE WHERE EXERCICE_IDT = new.EXERCICE_IDT), new.DATE_ECR, new.PERIODE) 
      INTO :REAL_PERIODE_DATE;

      --############# INFO GENERALE SUR LE CYCLE DES PERIODES
      SELECT m.PERIOD_CYCLE_MONTH, m.ANALYTIQUE_AUTO                      
      FROM MANDAT m

      INNER JOIN EXERCICE e
      ON e.EXERCICE_IDT = new.EXERCICE_IDT
      AND e.MANDAT_IDT = m.MANDAT_IDT
      INTO :PERIOD_CYCLE_MONTH, :ANALYTIQUE_AUTO;

      --############# CREATION DES RENSEIGNEMENTS SUR LES PERIODES
      i                 = new.periode;

      WHILE (i < (new.periode + new.repartition)) DO BEGIN
        INSERT INTO ECRITURES_ANALYTIQUE_PERIODE(EXERCICE_IDT, ECR_ID, PERIODE, PERIODE_DATE)
        VALUES(new.EXERCICE_IDT, new.ECR_ID, MOD(:i, :PERIOD_CYCLE_MONTH), :REAL_PERIODE_DATE);         
          
        i            = :i+1;
        REAL_PERIODE_DATE = dateadd(month, 1, :REAL_PERIODE_DATE);
      END

      --############# GENERATION AUTOMATIQUE DES ECRITURES ANALYTIQUES
      IF (:ANALYTIQUE_AUTO = 1) THEN BEGIN
        -- COMPTE
        FOR
            SELECT a.PERIODE, a.PERIODE_DATE
            FROM COMPTES c
                
            INNER JOIN ECRITURES_ANALYTIQUE_PERIODE a
            ON a.EXERCICE_IDT = new.EXERCICE_IDT
            AND a.ECR_ID = new.ECR_ID
                
            WHERE c.EXERCICE_IDT = new.EXERCICE_IDT
            AND c.cpte_id = new.cpte
            AND coalesce(c.budgetuse, 0) = 1
            INTO :i, :REAL_PERIODE_DATE
        DO BEGIN
            EXECUTE PROCEDURE ANALYTIQUE_SAVE_ECRITURES(    new.EXERCICE_IDT, new.DATE_ECR, new.ECR_ID,
                                                            new.CPTE_ANALYSE, new.CPTE_ANALYSE, :i,
                                                            new.DOSSIER, new.DEBIT/new.REPARTITION,
                                                            new.CREDIT/new.REPARTITION, new.CPTE,
                                                            :REAL_PERIODE_DATE);
        END

        -- CONTRE-PARTIE
        FOR
            SELECT a.PERIODE, a.PERIODE_DATE
            FROM COMPTES c
                
            INNER JOIN ECRITURES_ANALYTIQUE_PERIODE a
            ON a.EXERCICE_IDT = new.EXERCICE_IDT
            AND a.ECR_ID = new.ECR_ID
                
            WHERE c.EXERCICE_IDT = new.EXERCICE_IDT
            AND c.cpte_id = new.ccpte
            AND coalesce(c.budgetuse, 0) = 1
            INTO :i, :REAL_PERIODE_DATE
        DO BEGIN
            EXECUTE PROCEDURE ANALYTIQUE_SAVE_ECRITURES(    new.EXERCICE_IDT, new.DATE_ECR, new.ECR_ID,
                                                            new.CPTE_ANALYSE, new.CPTE_ANALYSE, :i,
                                                            new.DOSSIER, new.CREDIT/new.REPARTITION,
                                                            new.DEBIT/new.REPARTITION, new.CCPTE,
                                                            :REAL_PERIODE_DATE);
        END
      END
  end
end^

SET TERM ; ^